import { Logger } from "@sfroment/logger";
import { DRPNetworkNode, NetworkPb } from "@sfroment/network";
import { DRPObject } from "@sfroment/object";
import { drpMessagesHandler } from "./handlers.js";
import * as operations from "./operations.js";
import { DRPCredentialStore, DRPObjectStore } from "./store/index.js";
export let log;
export class DRPNode {
    config;
    objectStore;
    networkNode;
    credentialStore;
    constructor(config) {
        this.config = config;
        log = new Logger("drp::node", config?.log_config);
        this.networkNode = new DRPNetworkNode(config?.network_config);
        this.objectStore = new DRPObjectStore();
        this.credentialStore = new DRPCredentialStore(config?.credential_config);
    }
    async start() {
        await this.credentialStore.start();
        await this.networkNode.start();
        await this.networkNode.addMessageHandler(async ({ stream }) => drpMessagesHandler(this, stream));
        this.networkNode.addGroupMessageHandler("drp::topic::discovery", (e) => drpMessagesHandler(this, undefined, e.detail.msg.data).catch((e) => log.error("::DrpNode::Error while handling topic discovery message", e)));
    }
    async restart(config) {
        await this.networkNode.stop();
        this.networkNode = new DRPNetworkNode(config ? config.network_config : this.config?.network_config);
        await this.start();
    }
    addCustomGroup(group) {
        this.networkNode.subscribe(group);
    }
    addCustomGroupMessageHandler(group, handler) {
        this.networkNode.addGroupMessageHandler(group, handler);
    }
    async sendGroupMessage(group, data) {
        const message = NetworkPb.Message.create({
            sender: this.networkNode.peerId,
            type: NetworkPb.MessageType.MESSAGE_TYPE_CUSTOM,
            data,
        });
        await this.networkNode.broadcastMessage(group, message);
    }
    async addCustomMessageHandler(protocol, handler) {
        await this.networkNode.addCustomMessageHandler(protocol, handler);
    }
    async sendCustomMessage(peerId, data) {
        const message = NetworkPb.Message.create({
            sender: this.networkNode.peerId,
            type: NetworkPb.MessageType.MESSAGE_TYPE_CUSTOM,
            data,
        });
        await this.networkNode.sendMessage(peerId, message);
    }
    async createObject(options) {
        const object = new DRPObject({
            peerId: this.networkNode.peerId,
            publicCredential: options.acl ? undefined : this.credentialStore.getPublicCredential(),
            acl: options.acl,
            drp: options.drp,
            id: options.id,
        });
        operations.createObject(this, object);
        await operations.subscribeObject(this, object.id);
        if (options.sync?.enabled) {
            await operations.syncObject(this, object.id, options.sync.peerId);
        }
        return object;
    }
    /*
        Connect to an existing object
        @param options.id - The object ID
        @param options.drp - The DRP instance. It can be undefined
            where we just want the HG state
        @param options.sync.peerId - The peer ID to sync with
    */
    async connectObject(options) {
        const object = operations.connectObject(this, options.id, options.drp, options.sync?.peerId);
        return object;
    }
    async subscribeObject(id) {
        await operations.subscribeObject(this, id);
    }
    unsubscribeObject(id, purge) {
        operations.unsubscribeObject(this, id, purge);
        this.networkNode.removeTopicScoreParams(id);
    }
    async syncObject(id, peerId) {
        await operations.syncObject(this, id, peerId);
    }
}
